from datetime import timedelta, date, datetime

from logilab.mtconverter import html_escape

from cubicweb.schema import display_name
from cubicweb.utils import date_range, last_day, previous_month, next_month
from cubicweb.view import EntityView
from cubicweb.selectors import implements
from cubicweb.web.views.calendar import CalendarItemView, OneMonthCal
from cubicweb.web.views.old_calendar import _CalendarView

from cubes.calendar.entities import BadCalendar


CALENDARS_PAGE = u"""<table class="bigCalendars">
<tr><td class="calendar">%s</td></tr>
</table>
"""

WORKING_DAYS = ('working')
NON_WORKING_DAYS = ('non_working', 'conges_journee', 'maladie_journee',
                    'conges_mpaternite', 'greve_journee')

WORKING_AM_DAYS = ('working_am','maladie_apresmidi', 'conges_apresmidi',
                   'greve_apresmidi')
WORKING_PM_DAYS = ('working_pm','conges_matin', 'maladie_matin',
                   'greve_matin')

AM_TYPES = ('working_am','conges_matin', 'maladie_matin')
PM_TYPES = ('working_pm','maladie_apresmidi' ,'conges_apresmidi')

WORKING_HALF_DAYS = list(WORKING_AM_DAYS) + list(WORKING_PM_DAYS)


class ActivityMonthCalendarView(_CalendarView):
    """this viewca renders a 3x1 calendars' table"""
    id = 'calendaronemonth'
    title = _('calendar (one month)')

    def nav_header(self, curdate):
        self.req.add_js('cubicweb.ajax.js')
        prevdate = previous_month(curdate)
        nextdate = next_month(curdate)
        rql = self.req.url_quote(self.rset.printable_rql())
        prevlink = self.req.build_ajax_replace_url('onemonthcal', rql, 'calendaronemonth',
                                                   year=prevdate.year,
                                                   month=prevdate.month)
        nextlink = self.req.build_ajax_replace_url('onemonthcal', rql, 'calendaronemonth',
                                                   year=nextdate.year,
                                                   month=nextdate.month)
        self.w(u"""<table class="calendarPageHeader">
<tr><td class="prev"><a href="%s">&lt;&lt;</a></td>
<td class="next"><a href="%s">&gt;&gt;</a></td></tr>
</table>""" % (html_escape(prevlink), html_escape(nextlink)))


    def call(self, year=None, month=None):
        _today = date.today()
        year = int(self.req.form.get('year', _today.year))
        month = int(self.req.form.get('month', _today.month))
        center_date = date(year, month, 1)
        begin, end = self.get_date_range(day=center_date, shift=0)
        schedule = self._mk_schedule(begin, end)
        calendars = self.build_calendars(schedule, begin, end)
        self.w(u'<div id="onemonthcal">')
        self.nav_header(center_date)
        self.w(CALENDARS_PAGE % tuple(calendars))
        self.w(u'</div>')


def get_date_range_from_req(req, year=None, month=None):
    """return (firstday, lastday)"""
    year = year or req.form.get('year')
    month = month or req.form.get('month')
    if year and month:
        year = int(year)
        month = int(month)
        firstday = date(year, month, 1)
    else:
        _today = datetime.today()
        firstday = date(_today.year, _today.month, 1)
    return firstday, last_day(firstday)

class EuserMonitorCalendar(EntityView):
    id = 'user_activity_calendar'
    __select__ = implements('CWUser',)

    def cell_call(self, row, col, year=None, month=None, calid=None):
        self.req.add_js('cubicweb.ajax.js')
        self.req.add_css('cubes.calendar.css')
        user = self.entity(row, col)
        calid = calid or 'tid%s' % user.eid
        firstday, lastday = get_date_range_from_req(self.req, year, month)

        # make calendar
        caption = '%s %s' % (self.req._(firstday.strftime('%B').lower()), firstday.year)
        prevurl, nexturl = self._prevnext_links(firstday, user, calid)
        prevlink = '<a href="%s">&lt;&lt;</a>' % html_escape(prevurl)
        nextlink = '<a href="%s">&gt;&gt;</a>' % html_escape(nexturl)

        # build cells
        try:
            celldatas = self._build_activities(user, firstday, lastday)
        except BadCalendar, exc: # in case of missing week day information
            self.w(u'<div class="error">%s</div>' % exc)
            return
        # build table/header
        self.w(u'<table id="%s" class="activitiesCal">'
               u'<tr><th class="prev">%s</th>'
               u'<th class="calTitle" colspan="5"><span>%s</span></th>'
               u'<th class="next">%s</th></tr>'
               u'<tr><th>L</th><th>M</th><th>M</th><th>J</th><th>V</th><th>S</th><th>D</th></tr>'
               % (calid, prevlink, caption, nextlink))
        start = firstday - timedelta(firstday.weekday())
        stop = lastday + timedelta(6 - lastday.weekday())
        for curdate in date_range(start, stop):
            if curdate == start or curdate.weekday() == 0: # sunday
                self.w(u'<tr>')
            self._build_calendar_cell(curdate, celldatas, firstday)
            if curdate.weekday() == 6:
                self.w(u'</tr>')
        self.w(u'</table>')

    def _build_calendar_cell(self, curdate, celldatas, firstday):
        if curdate.month != firstday.month:
            self.w(u'<td class="outofrange"></td>')
        else:
            cssclasses, total_duration, url, workcases = celldatas[curdate]
            total = total_duration
            if workcases:
                total = u"%s (%s)" % (total, workcases)
            cellcontent = u'<a title="total %s" href="%s">%s</a>' % (
                total, url, curdate.day)
            self.w(u'<td class="%s">%s</td>' % (u' '.join(cssclasses),  cellcontent))

    def _prevnext_links(self, curdate, user, calid):
        prevdate = previous_month(curdate)
        nextdate = next_month(curdate)
        rql = 'Any X WHERE X eid %s' % user.eid
        prevlink = self.req.build_ajax_replace_url(calid, rql, 'user_activity_calendar',
                                                   replacemode='swap',
                                                   month=prevdate.month, year=prevdate.year)
        nextlink = self.req.build_ajax_replace_url(calid, rql, 'user_activity_calendar',
                                                   replacemode='swap',
                                                   month=nextdate.month, year=nextdate.year)
        return prevlink, nextlink

    def _build_activities(self, user, firstday, lastday):
        # figure out which days of month are working days
        resource = user.default_resource
        if resource:
            day_types = dict((date, (dtype, state)) for date, dtype, state in
                             user.reverse_euser[0].get_day_types(firstday, lastday))
        else:
            day_types = {}
        working_dtype = {}
        for eid, state in set(day_types.values()):
            working_dtype[eid] = self.req.eid_rset(eid).get_entity(0, 0)

        # get activities
        activities = {}
        workcases = {}
        pending_activities = set()
        rql = ("Any DI,DU,S,WR WHERE A is Activity, "
               "  A in_state ST, ST name S, "
               "  A diem <= %(l)s, A diem >= %(f)s, "
               "  A diem DI, A duration DU, "
               "  A done_by R, R euser U, U eid %(u)s, "
               "  A done_for WO, W split_into WO, W title WR")
        rset = self.req.execute(rql, {'u': user.eid, 'l': lastday, 'f': firstday})
        for diem, duration, state, workcase in rset:
            activities.setdefault(diem, 0)
            workcases.setdefault(diem, [])
            activities[diem] += float(duration)
            workcases[diem].append(workcase)
            if state == 'pending':
                pending_activities.add(diem)

        # build result
        celldatas = {}
        _today = date.today()
        for date_ in date_range(firstday, lastday):
            cssclass = []
            total_duration = activities.get(date_, 0.)
            dtype, state = day_types.get(date_, (None, None))
            if state == 'pending':
                cssclass.append(u'daytype_pending')
            if dtype and working_dtype[dtype].work_during:
                if date_ <= _today:
                    dtype_title = working_dtype[dtype].title
                    if ((total_duration != 1. and  dtype_title in WORKING_DAYS) or
                        (total_duration != 0.5 and dtype_title in WORKING_HALF_DAYS)):
                        cssclass.append(u'problem')
                    else:
                        if date_ in pending_activities:
                            cssclass.append(u'pending')
                        else:
                            cssclass.append(u'completed')
                    if dtype_title in WORKING_DAYS:
                        total_duration = u'%s / 1.0' % total_duration
                    elif dtype_title in WORKING_HALF_DAYS:
                        total_duration = u'%s / 0.5' % total_duration
                else:
                    if total_duration:
                        cssclass.append(u'problem')
            else:
                if state != 'pending':
                    cssclass.append(u'closed')
                if total_duration:
                    cssclass.append(u'problem')
            a_rql = ("Any A WHERE A is Activity, A diem = '%(d)s', "
                     "A done_by R, R euser U, U eid %(u)s"
                     % {'u': user.eid, 'd': date_.strftime('%Y-%m-%d')})
            url = html_escape(self.build_url(rql=a_rql, vid='activities-submit',
                                             year=date_.year, month=date_.month, day=date_.day))
            workcases_str = ','.join(workcases.get(date_, ''))
            celldatas[date_] = (cssclass, total_duration, url, workcases_str)
        if _today in celldatas:
            # celldatas maps days to tuples (cssclass, duration, url, descr)
            celldatas[_today][0].append(u'today')
        return celldatas


class EuserCalendar(EntityView):
    __select__ = implements('Calendar',)
    id = 'user_calendar'

    def cell_call(self, row, col):
        self.req.add_css('cubes.calendar.css')
        entity = self.entity(row, col)
        _today = datetime.today()
        year = int(self.req.form.get('year', _today.year))
        month = int(self.req.form.get('month', _today.month))
        firstday, lastday = get_date_range_from_req(self.req, year, month)
        # make calendar
        caption = '%s %s' % (self.req._(firstday.strftime('%B').lower()), firstday.year)
        prevurl, nexturl = self._prevnext_links(entity,firstday)
        prevlink = '<a href="%s">&lt;&lt;</a>' % html_escape(prevurl)
        nextlink = '<a href="%s">&gt;&gt;</a>' % html_escape(nexturl)
        self.w(u'<table id="ctid%s" class="userCal">'
               u'<tr><th class="prev">%s</th>'
               u'<th class="calTitle" colspan="6"><span>%s</span></th>'
               u'<th class="next">%s</th></tr>'
               u'<tr><th>&nbsp;</th><th>L</th><th>M</th><th>M</th><th>J</th><th>V</th><th>S</th><th>D</th></tr>'
               % (entity.eid, prevlink, caption, nextlink))
        start = firstday - timedelta(firstday.weekday())
        stop = lastday + timedelta(6 - lastday.weekday())
        celldatas = self._build_celldata(entity, firstday, lastday)
        # build cells
        for curdate in date_range(start, stop):
            if curdate == start or curdate.weekday() == 0: # sunday
                self.w(u'<tr>')
                self.w(u'<td class="week">%s<br/> %d</td>' % (self.req._('week'), curdate.isocalendar()[1]))
            self._build_calendar_cell(curdate, celldatas, firstday)
            if curdate.weekday() == 6:
                self.w(u'</tr>')
        self.w(u'</table>')

    def _prevnext_links(self, entity, curdate):
        prevdate = previous_month(curdate)
        nextdate = next_month(curdate)
        rql = 'Any X WHERE X eid %s' % entity.eid
        prevlink = self.req.build_ajax_replace_url('ctid%s' % entity.eid, rql,
                                                   'user_calendar', replacemode='swap',
                                                   month=prevdate.month, year=prevdate.year)
        nextlink = self.req.build_ajax_replace_url('ctid%s' % entity.eid, rql, 'user_calendar',
                                                   replacemode='swap',
                                                   month=nextdate.month, year=nextdate.year)
        return prevlink, nextlink

    def _build_calendar_cell(self, curdate, celldatas, firstday):
        if curdate.month != firstday.month:
            self.w(u'<td class="outofrange"></td>')
        else:
            cssclasses, total_duration, url, workcases = celldatas[curdate]
            stickers = u' '.join(cssclasses)
            pending = self.req._('pending')

            klasses=[]
            am_stickers=[]
            pm_stickers=[]
            for cssclass in cssclasses:
                klass = cssclass.split(':')[0]
                pending_suffix = cssclass.endswith(':pending') and '_pending' or ''
                if curdate == datetime.today():
                    klasses.append(u'today')
                if klass in NON_WORKING_DAYS:
                    klasses.append(u'non_working'+pending_suffix)
                    am_stickers.append(self.req._(klass))
                    pm_stickers.append(self.req._(klass))
                elif klass in WORKING_PM_DAYS:
                    klasses.append(u'working_pm'+pending_suffix)
                    if klass in AM_TYPES:
                        am_stickers.append(self.req._(klass))
                        pm_stickers.append(self.req._('working'))
                    else:
                        am_stickers.append(self.req._('non_working'))
                        pm_stickers.append(self.req._(klass))
                elif klass in WORKING_AM_DAYS:
                    klasses.append(u'working_am'+pending_suffix)
                    if klass in AM_TYPES:
                        am_stickers.append(self.req._(klass))
                        pm_stickers.append(self.req._('non_working'))
                    else:
                        am_stickers.append(self.req._('working'))
                        pm_stickers.append(self.req._(klass))
                elif klass in WORKING_DAYS:
                    klasses.append(u'working'+pending_suffix)
                    am_stickers.append(self.req._(klass))
                    pm_stickers.append(self.req._(klass))
                else:
                    self.error('Unknown daytype class %s', klass)
            am_cell ='<tr><td class="am"><div class="mday">am</div><div class="stickers">%s</div></td></tr>\n' % (' '.join(am_stickers))
            pm_cell ='<tr><td class="pm"><div class="mday">pm</div><div class="stickers">%s</div></td></tr>\n' % (' '.join(pm_stickers))
            cell_url = u'<a title="total %s" href="%s">%s</a>' % (total_duration, url, curdate.day)
            # FIXME cell_url
            cellcontent = u'<table class="caldata">'
            cellcontent +='<tr><td class="caldate">%s</td></tr>%s %s</table>' % (cell_url,  am_cell, pm_cell)
            self.w(u'<td class="%s" title="%s">%s</td>' %(' '.join(klasses), html_escape(stickers),  cellcontent))

    def _build_celldata(self, entity, firstday, lastday):
        celldatas = {}
        for date, dtype, state in entity.get_days_type(firstday, lastday):
            cssclass = []
            if dtype is None:
                cssclass.append(u'unknown')
            else:
                dtype = self.req.eid_rset(dtype).get_entity(0, 0)
                if state == 'pending':
                    cssclass.append('%s:%s' % (dtype.title,state))
                else:
                    cssclass.append(dtype.title)
            celldatas[date] = (cssclass, 0, '', '')
        return celldatas

class TimeperiodCalendarItemView(EntityView):
    id = 'calendaritem'
    __select__ = implements('Timeperiod',)
    def cell_call(self, row, col):
        entity = self.complete_entity(row, col)
        day_types = u' '.join(self.req._(dt.title) for dt in entity.day_type)
        calendar = entity.from_calendar
        self.w(html_escape(u'%s : %s (%s)' % (calendar.title, day_types, self.req._(entity.state))))



"""template-specific forms/views/actions/components"""
from cubicweb.web import uicfg

uicfg.autoform_section.tag_object_of(('*', 'periods', 'Timeperiod'), 'primary')

uicfg.primaryview_section.tag_subject_of(('Calendar', 'weekdays', '*'), 'hidden')
uicfg.primaryview_section.tag_subject_of(('Calendar', 'days', '*'), 'hidden')
uicfg.primaryview_section.tag_subject_of(('Calendar', 'periods', '*'), 'hidden')
uicfg.primaryview_section.tag_subject_of(('Calendar', 'day_types', '*'), 'hidden')
uicfg.primaryview_section.tag_subject_of(('Calendar', 'inherits', '*'), 'hidden')

for rtype in ('days', 'weekdays', 'periods', 'day_types', 'inherits'):
    uicfg.actionbox_appearsin_addmenu.tag_subject_of(('Calendar', rtype, '*'), True)

uicfg.actionbox_appearsin_addmenu.tag_subject_of(('Daytype', 'work_during', '*'), True)

uicfg.actionbox_appearsin_addmenu.tag_object_of(('Calendar', 'inherits', '*'), True)
uicfg.actionbox_appearsin_addmenu.tag_object_of(('*', 'for_calendar', 'Calendar'), True)
uicfg.actionbox_appearsin_addmenu.tag_object_of(('*', 'for_period', 'Timeperiod'), True)




from cubicweb.selectors import implements
from cubicweb.web.facet import RelationFacet

class TimeperiodCalendarFacet(RelationFacet):
    id = 'timeperiod-calendar-facet'
    __select__ = RelationFacet.__select__ & implements('Timeperiod',)
    rtype = 'periods'
    target_attr = 'title'



# cube's specific schema
from yams.buildobjs import EntityType, SubjectRelation, String, Datetime, Time

from cubicweb.schema import WorkflowableEntityType

_ = unicode

class Calendar(EntityType):
    """see projman"""
    title = String(required=True, unique=True, maxsize=64)
    weekdays = SubjectRelation('WeekDay', cardinality='**')
    days = SubjectRelation('Recurrentday', cardinality='**')
    periods = SubjectRelation('Timeperiod', cardinality = '*1', composite='subject')
    day_types = SubjectRelation('Daytype', cardinality='**')
    inherits = SubjectRelation('Calendar', cardinality='?*')

class Calendaruse(EntityType):
    """see projman"""
    start = Datetime()
    stop = Datetime()
    use_calendar = SubjectRelation('Calendar', cardinality='1*')

class Daytype(EntityType):
    """see projman"""
    title = String(required=True, maxsize=32, internationalizable=True)
    work_during = SubjectRelation('Timespan', cardinality='**')

class Timespan(EntityType):
    """see projman"""
    start = Time(required=True)
    stop = Time(required=True)

class Timeperiod(WorkflowableEntityType):
    """ """
    start = Datetime(required=True)
    stop = Datetime(required=True)
    day_type = SubjectRelation('Daytype', cardinality='1*')

class WeekDay(EntityType):
    day_of_week = String(vocabulary=[_('monday'), _('tuesday'), _('wednesday'),
                                     _('thursday'), _('friday'), _('saturday'),
                                     _('sunday')],
                         internationalizable=True)
    day_type = SubjectRelation('Daytype', cardinality='1*')


class Recurrentday(EntityType):
    day_month = String(description='mm-dd', required=True, maxsize=5)
    day_type = SubjectRelation('Daytype', cardinality='1*')



from logilab.common.testlib import unittest_main
from cubicweb.devtools.testlib import AutomaticWebTest

class AutomaticWebTest(AutomaticWebTest):

    def to_test_etypes(self):
        return set(('Calendar', 'Calendaruser', 'Daytype',
                    'Timespan', 'Timeperiod',
                    'WeekDay', 'Recurrentday',
                    'CWUser'))

    def list_startup_views(self):
        return ()

if __name__ == '__main__':
    unittest_main()



from cubicweb.utils import date_range, todate, todatetime
from cubicweb.interfaces import ICalendarable
from cubicweb.entities import AnyEntity, fetch_config

class BadCalendar(ValueError): pass

def intersect(a,b):
    c = (max(a[0],b[0]), min(a[1], b[1]))
    return c[0] <= c[1]

class Timespan(AnyEntity):
    id = 'Timespan'
    fetch_attrs, fetch_order = fetch_config(('start', 'stop'))

    def dc_title(self):
        return u'%s - %s' % (self.printable_value('start'),
                             self.printable_value('stop'))

def use_class_cache(cachename, cachepart):
    def cached(func):
        def wrapper(self, *args):
            cache = getattr(self.__class__, cachename).setdefault(cachepart,{})
            cachekey = (self.eid,) + args
            try:
                res = cache[cachekey]
            except KeyError:
                res = func(self, *args)
                cache[cachekey] = res
            return res
        return wrapper
    return cached

class Timeperiod(AnyEntity):
    id='Timeperiod'
    fetch_attrs, fetch_order = fetch_config(('start', 'stop'))
    __implements__ = (ICalendarable,)
    conges_days = [u'conges_journee', u'conges_matin', u'conges_apresmidi']

    def dc_title(self):
        return u'%s-%s' % (self.format_date(self.start),
                           self.format_date(self.stop))

    def dc_long_title(self):
        day_types = u' '.join(self.req._(dt.title) for dt in self.day_type)
        return u'%s : %s (%s)' % (self.dc_title(), day_types, self.req._(self.state))

    @property
    def from_calendar(self):
        return self.reverse_periods[0]

class Calendar(AnyEntity):
    id = 'Calendar'
    rest_attr = 'title'
    fetch_attrs, fetch_order = fetch_config(('title',))
    daytype_cache = {}

    def dc_title(self):
        return self.title

    def parent(self):
        parents = self.inherits
        if parents:
            return parents[0]

    def _fill_cache(self, cache, start, stop):
        # prepare
        periods = self._get_timeperiods()
        rec_days, week_days = self._get_rec_weekdays()
        if self.inherits:
            parent = dict([(item[0], item) for item in self.inherits[0].get_days_type(start, stop)])
        else:
            parent = {}
        # iterate
        for date in date_range(start, stop):
            key = (self.eid, date)
            value = None
            # try period
            for pa, po, ptype, state in periods:
                if pa <= todatetime(date) <= po:
                    value = (self.eid, date, ptype, state)
                    break
            else:
                # try recurrent day
                eid = rec_days.get(date.strftime('%m-%d'))
                if eid is not None:
                    value = (self.eid, date, eid, 'validated')
                else:
                    # try week day
                    eid = week_days.get(date.weekday())
                    if eid is not None:
                        value = (self.eid, date, eid, 'validated')
                    else:
                        if date in parent:
                            value = (self.eid,) + parent[date]
                        else:
                            raise BadCalendar('bad calendar definition, '
                                              'missing week days '
                                              'in root calendar for %s'
                                              % self.eid)
            cache[key] = value

    @use_class_cache('daytype_cache','periods')
    def _get_timeperiods(self):
        rql = ('Any PA,PO,D,SN WHERE C eid %(c)s, C periods P, P day_type D, '
               'P start PA, P stop PO, P in_state S, S name SN')
        rset = self.req.execute(rql, {'c': self.eid})
        return tuple(rset)

    @use_class_cache('daytype_cache','recweek')
    def _get_rec_weekdays(self):
        # recurrent days
        rql = 'Any D,DM WHERE C eid %(c)s, C days R, R day_month DM, R day_type D'
        rset = self.req.execute(rql, {'c': self.eid})
        rec_days = dict((row[1], row[0]) for row in rset)
        # week days
        DAYS = [u'monday', u'tuesday', u'wednesday', u'thursday', u'friday', u'saturday', u'sunday']
        rql = 'Any D,W WHERE C eid %(c)s, C weekdays R, R day_of_week W, R day_type D'
        rset = self.req.execute(rql, {'c': self.eid})
        week_days = dict((DAYS.index(day), eid) for eid, day in rset)
        return rec_days, week_days

    @use_class_cache('daytype_cache','days_type')
    def _get_days_type(self, start, stop):
        """return [(cal_eid, date, day_type_eid, state), ...]
        with state in ('pending', 'validated')
        """
        day_types = []
        cache = self.daytype_cache.setdefault('cal_date', {})
        for date in date_range(start, stop):
            key = (self.eid, todate(date))
            if key not in cache:
                for hole in date_range(date, stop):
                    if (self.eid, hole) in cache:
                        break
                self._fill_cache(cache, date, hole)
            day_types.append(cache[key])
        return day_types

    def _get_daytype_from_titles(self, dtypes):
        """map a set of titles to a set of eids"""
        assert isinstance(dtypes, frozenset), 'dtypes should be a frozenset'
        cache = self.daytype_cache.setdefault('day_title',{})
        if dtypes not in cache:
            types = tuple(dtypes)
            if len(types) == 1:
                rset = self.req.execute('Any D WHERE D is Daytype, D title %(title)s',
                                        {'title': types[0]})
            else:
                rset = self.req.execute('Any D WHERE D is Daytype, D title IN %s'
                                        % str(tuple(types)))
            cache[dtypes] = frozenset(row[0] for row in rset)
        return cache[dtypes]

    def get_days_type(self, start, stop, dtypes=None):
        """return [(date, day_type_eid, state), ...] with
        date varying from start to stop"""
        days_type = self._get_days_type(start, stop)
        if dtypes is None:
            return [item[1:] for item in days_type]
        dtype_eids = self._get_daytype_from_titles(frozenset(dtypes))
        return [item[1:] for item in days_type if item[2] in dtype_eids]

class Daytype(AnyEntity):
    id = 'Daytype'
    fetch_attrs, fetch_order = fetch_config(('title',))

class CalendarUse(AnyEntity):
    id = 'Calendaruse'
    fetch_attrs, fetch_order = fetch_config(('start', 'stop'))

    @property
    def calendar(self):
        return self.use_calendar[0]

class RecurrentDay(AnyEntity):
    id = 'Recurrentday'
    fetch_attrs, fetch_order = fetch_config(('day_month', 'day_type'))

    def dc_title(self):
        return self.req._(self.day_month)

    def dc_long_title(self):
        day_types = u' '.join(dt.title for dt in self.day_type)
        return u'%s : %s' % (self.dc_title(), day_types)

class WeekDay(AnyEntity):
    id = 'WeekDay'
    fetch_attrs, fetch_order = fetch_config(('day_of_week', 'day_type'))

    def dc_title(self):
        return self.req._(self.day_of_week)

    def dc_long_title(self):
        day_types = u' '.join(dt.title for dt in self.day_type)
        return u'%s : %s' % (self.dc_title(), day_types)




from logilab.mtconverter import html_escape

from cubicweb.view import EntityView
from cubicweb.selectors import implements
from cubicweb.web.views import baseviews

# out of context  ####################################################################


class RecurrentDayOutOfContext(baseviews.OutOfContextView):
    __select__ = implements('Recurrentday')

    def cell_call(self, row, col):
        entity = self.entity(row,col)
        self.w(u'<a href="%s">' % html_escape(entity.absolute_url(vid='edition')))
        self.wdata(entity.dc_long_title())
        self.w(u'</a>')

class TimePeriodOutOfContext(baseviews.OutOfContextView):
    __select__ = implements('Timeperiod')

    def cell_call(self, row, col):
        entity = self.entity(row,col)
        self.w(u'<a href="%s">' % html_escape(entity.absolute_url(vid='edition')))
        self.wdata(entity.dc_long_title())
        self.w(u'</a>')





# postcreate script. You could setup a workflow here for example

## TIMESPANS
morning = rql('INSERT Timespan TS: TS start "08:00:00", TS stop "12:00:00"').rows[0][0]
afternoon = rql('INSERT Timespan TS: TS start "13:00:00", TS stop "17:00:00"').rows[0][0]

## DAY TYPES
# register day types to i18n catalog
_('working'), _('non_working'), _('working_am'), _('working_pm')
# insertion of Daytype entities
dtype_working = rql('INSERT Daytype DT: DT title "working", DT work_during TM, DT work_during TA '
                    'WHERE TM eid %(tm)s, TA eid %(ta)s',
                    {'tm' : morning, 'ta' : afternoon}).rows[0][0]
dtype_nonworking = rql('INSERT Daytype DT: DT title "non_working"').rows[0][0]
dtype_working_am = rql('INSERT Daytype DT: DT title "working_am", '
                       'DT work_during TA WHERE TA eid %(ta)s',
                       {'ta' : morning}).rows[0][0]
dtype_working_pm = rql('INSERT Daytype DT: DT title "working_pm", '
                       'DT work_during TP WHERE TP eid %(tp)s',
                       {'tp' : afternoon}).rows[0][0]

## WEEK DAYS
for day in (u'monday', u'tuesday', u'wednesday', u'thursday', u'friday'):
    rql('INSERT WeekDay WD: WD day_of_week %(day)s, WD day_type DT '
        'WHERE DT eid %(dt)s',
        {'day' : day, 'dt' : dtype_working})

for day in (u'saturday', u'sunday'):
    rql('INSERT WeekDay WD: WD day_of_week %(day)s, WD day_type DT '
        'WHERE DT eid %(dt)s',
        {'day': day, 'dt' : dtype_nonworking})

## DEFAULT CALENDAR
defaultcal = rql('INSERT Calendar C: C title "Calendrier Francais"').rows[0][0]

rql('SET C weekdays WD WHERE WD is WeekDay, C eid %(cal)s', {'cal' : defaultcal})
rql('SET C day_types DT WHERE DT is Daytype, C eid %(cal)s', {'cal' : defaultcal})

# Timeperiod workflow
wf = add_workflow('time-period workflow', 'Timeperiod')
tp_pending = wf.add_state(_('pending'), initial=True)
tp_validated  = wf.add_state(_('validated'))
wf.add_transition(_('validate'), tp_pending, tp_validated, ('managers',))




from cubicweb.selectors import implements
from cubicweb.web.views import primary
from cubicweb.common.uilib import toggle_link

# calendar  ######################################################################

class PrimaryCalendar(primary.PrimaryView):
    __select__ = implements('Calendar')

    def render_entity_attributes(self, entity):
        entity.view('user_calendar', w=self.w)
        self._show_periods(entity)
        self._show_recurrent_days(entity)
        self._show_week_days(entity)
        self._show_day_types(entity)
        self._show_children(entity)

    def _show_periods(self, calendar):
        items = calendar.related('periods')
        title = u'%s (%i)' % (_(u'Periods'), len(items))
        self.w(u'<h2 class="cal_properties">%s</h2>\n' %
               toggle_link('cal_pd', title))
        self.wview('list', items, 'null', klass='hidden', listid='cal_pd')

    def _show_recurrent_days(self, calendar):
        items = calendar.related('days')
        title = u'%s (%i)' % (_(u'Recurrent days'), len(items))
        self.w(u'<h2 class="cal_properties">%s</h2>\n' %
               toggle_link('cal_rd', title))
        self.wview('list', items, 'null', klass='hidden', listid='cal_rd')

    def _show_week_days(self, calendar):
        items = calendar.related('weekdays')
        title = u'%s (%i)' % (_(u'Week days'), len(items))
        self.w(u'<h2 class="cal_properties">%s</h2>\n' %
               toggle_link('cal_wd', title))
        self.wview('list', items, 'null', klass='hidden', listid='cal_wd')

    def _show_day_types(self, calendar):
        items = calendar.related('day_types')
        title = u'%s (%i)' % (_(u'Day types'), len(items))
        self.w(u'<h2 class="cal_properties">%s</h2>\n' %
               toggle_link('cal_dt', title))
        self.wview('list', items, 'null', klass='hidden', listid='cal_dt')

    def _show_children(self, calendar):
        items = calendar.related('inherits', 'object')
        title = u'%s (%i)' % (_(u'inherits_object'), len(items))
        self.w(u'<h2 class="cal_properties">%s</h2>\n' %
               toggle_link('cal_ch', title))
        self.wview('list', items, 'null', klass='hidden', listid='cal_ch')



# pylint: disable-msg=W0622
"""cubicweb-calendar application packaging information"""

modname = 'calendar'
distname = 'cubicweb-calendar'

numversion = (0, 2, 0)
version = '.'.join(str(num) for num in numversion)

license = 'LGPL'
copyright = '''Copyright (c) 2008 LOGILAB S.A. (Paris, FRANCE).
http://www.logilab.fr/ -- mailto:contact@logilab.fr'''

author = 'Logilab'
author_email = 'contact@logilab.fr'

web = 'http://www.cubicweb.org/project/%s' % distname

short_desc = 'calendar component for the CubicWeb framework'
long_desc = """This CubicWeb component models calendars with different types of days
(working, non-working, vacation, sick, etc) and time periods (from simple
"Aug 31st 2009 to Sep 4th 2009" to repetitive ones like "July 14th").

CubicWeb is a semantic web application framework, see http://www.cubicweb.org
"""

classifiers = [
    'Environment :: Web Environment',
    'Framework :: CubicWeb',
    'Programming Language :: Python',
    'Programming Language :: JavaScript',
    ]

__depends_cubes__ = {}
__depends__ = {'cubicweb': '>= 3.5.0'}
__recommend__ = ()

__use__ = tuple(__depends_cubes__)

# packaging

from os import listdir as _listdir
from os.path import join, isdir

pyversions = ['2.4']

#from cubicweb.devtools.pkginfo import get_distutils_datafiles
CUBES_DIR = join('share', 'cubicweb', 'cubes')
THIS_CUBE_DIR = join(CUBES_DIR, 'calendar')

def listdir(dirpath):
    return [join(dirpath, fname) for fname in _listdir(dirpath)
            if fname[0] != '.' and not fname.endswith('.pyc')
            and not fname.endswith('~')]

from glob import glob
try:
    data_files = [
        # common files
        [THIS_CUBE_DIR, [fname for fname in glob('*.py') if fname != 'setup.py']],
        ]
    # check for possible extended cube layout
    for dirname in ('entities', 'views', 'sobjects', 'hooks', 'schema', 'data', 'i18n', 'migration'):
        if isdir(dirname):
            data_files.append([join(THIS_CUBE_DIR, dirname), listdir(dirname)])
    # Note: here, you'll need to add subdirectories if you want
    # them to be included in the debian package
except OSError:
    # we are in an installed directory
    pass

cube_eid = None # <=== FIXME if you need direct bug-subscription
